<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class SolicitacaoFactory
{

    use TQuery;

    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getLocal()
    {
        $sql = "SELECT NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRTIPOLOCALCLI, FCM_TIPOLOCALCLI.IDTIPOLOCALCLI
                    FROM FCM_LOCALCLI
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_LOCALCLI.NRTIPOLOCALCLI
                    WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = '3'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertSolicitacao($parametros)
    {

        $sql = "INSERT INTO FCM_SOLICITACAO (NRSOLICITACAO, NMSOLICITACAO, DTSOLICITACAO, IDSITUACAO, NRLOCALCLIENTE,
            NRUSUARIOAPP, DSSOLICITACAO, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, NRORGULTATU,DTULTATU)
              VALUES (:NRSOLICITACAO, :NMSOLICITACAO, SYSDATE, :IDSITUACAO, :NRLOCALCLIENTE, :NRUSUARIOAPP,
                      :DSSOLICITACAO, :NRORG, 'S', SYSDATE,  :NRORGINCLUSAO, :NRORGULTATU, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertHistorico($parametros)
    {
       
        $sql = "INSERT INTO FCM_SOLICITACAOHISTORICO (NRSOLICITACAOHISTORICO, NRSOLICITACAO, NMSOLICITACAO, DTHISTSOLICITACAO,
                IDSITUACAO, IDTIPOHISTORICO, NRUSUARIOAPPSOLICITANTE, DSSOLICITACAO, IDSATISFACAOSOLICITA, NRORG, IDATIVO, DTULTATU)
                VALUES (:NRSOLICITACAOHISTORICO, :NRSOLICITACAO, :NMSOLICITACAO, SYSDATE, :IDSITUACAO, :IDTIPOHISTORICO,
              :NRUSUARIOAPPSOLICITANTE, :DSSOLICITACAO, :IDSATISFACAOSOLICITA, :NRORG, 'S', SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getSolicitacoes($userId, $filter)
    {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO,  S.DSSOLICITACAO, S.NRLOCALCLIENTE, LC.NMLOCALCLIENTE, S.DTSOLICITACAO,
                       TO_CHAR(S.DTSOLICITACAO, 'DD/MM') DTAGEND, TO_CHAR(S.DTSOLICITACAO, 'HH24:MI') HORAAGEND, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI,
                  CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                 FROM FCM_SOLICITACAO S
                 JOIN FCM_LOCALCLI LC
                   ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                 JOIN FCM_LOCALCLI LCP
                  ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                WHERE S.NRUSUARIOAPP IN ({$userId})" . $filter . "
                ORDER BY S.DTSOLICITACAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function showRequest($parametros)
    {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO, S.DSSOLICITACAO, TO_CHAR(S.DTSOLICITACAO, 'DD/MM HH24:MI') DTSOLICITACAO, S.NRLOCALCLIENTE, S.IDSITUACAO, LC.NMLOCALCLIENTE,
                    CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                FROM FCM_SOLICITACAO S
                JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                WHERE S.NRSOLICITACAO = :NRSOLICITACAO";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function upRequest($parametros)
    {

        $sql = "UPDATE FCM_SOLICITACAO
                  SET IDSITUACAO = :IDSITUACAO,
                      DSSOLICITACAO = :DSSOLICITACAO
                WHERE NRSOLICITACAO = :NRSOLICITACAO";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertImg($param)
    {

        $sql = "INSERT INTO FCM_IMGSOLICITACAO (NRIMGSOLICITACAO, NRSOLICITACAO, DSCAMINHO, NRORG, IDATIVO, DSTYPE, NRSIZE, DTINCLUSAO)
              VALUES (:NRIMGSOLICITACAO, :NRSOLICITACAO, :DSCAMINHO, :NRORG, 'S', :DSTYPE, :NRSIZE,  SYSDATE)";

        $this->connection->executeStatement($sql, $param);
    }

    public function getFotosSolicitacao($param)
    {

        $sql = "SELECT NRIMGSOLICITACAO, DSCAMINHO
                  FROM FCM_IMGSOLICITACAO
                 WHERE NRSOLICITACAO = " . $param['NRSOLICITACAO'];

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getAvaliacao($parametros)
    {
        $sql = "SELECT IDSATISFACAOSOLICITA FROM FCM_SOLICITACAOHISTORICO WHERE NRSOLICITACAO = :NRSOLICITACAO AND IDSATISFACAOSOLICITA IS NOT NULL";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getTarefasVinculadas($parametros)
    {
        $sql = "SELECT FCM_SOLICITACAOTAREFAS.NRCHECKLISTAGEND
                    FROM FCM_SOLICITACAOTAREFAS
                    JOIN FCM_CHECKLISTAGEND
                    ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_SOLICITACAOTAREFAS.NRCHECKLISTAGEND
                    WHERE NRSOLICITACAO = :NRSOLICITACAO";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function setStatusTarefas($parametros)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                SET IDSITUACAOAGEND = :IDSITUACAOAGEND,
                    CDOPERULTATU = :CDOPERULTATU,
                    DTULTATU = SYSDATE
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }
}
