<?php

namespace FM\Service;

use FM\Factory\SolicitacaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;
use FM\Service\FindService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class SolicitacaoService
{

    protected $SolicitacaoFactory;
    protected $FindService;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $FileServerService;

    public function __construct(SolicitacaoFactory $SolicitacaoFactory, FindService $FindService, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FileServerService $FileServerService)
    {
        $this->SolicitacaoFactory           = $SolicitacaoFactory;
        $this->FindService                  = $FindService;
        $this->environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
        $this->FileServerService            = $FileServerService;
    }

    public function getLocal()
    {
        return $this->SolicitacaoFactory->getLocal();
    }


    public function criarSolicitacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = 20;
            $session = InstanceManager::getInstance()->getService('session');
            $session->start();
            $usuario = $session->get('NRUSUARIOAPP');

            $NRSOLICITACAO = $this->newCode->getCodeByTableName("FCM_SOLICITACAO", $NRORG, 1, 10, true);

            $this->SolicitacaoFactory->insertSolicitacao(array(
                "NRSOLICITACAO"  => $NRSOLICITACAO,
                "NMSOLICITACAO"  => $parametros['NMSOLICITACAO'],
                "IDSITUACAO"     => $parametros['IDSITUACAO'],
                "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                "NRUSUARIOAPP"   => $usuario,
                "DSSOLICITACAO"  => $parametros['DSSOLICITACAO'],
                "NRORG"          => $NRORG,
                "NRORGINCLUSAO"  => $NRORG,
                "NRORGULTATU"    => $NRORG,
            ));

            $this->SolicitacaoFactory->insertHistorico(array(
                "NRSOLICITACAOHISTORICO"    => $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true),
                "NRSOLICITACAO"             => $NRSOLICITACAO,
                "NMSOLICITACAO"             => $parametros['NMSOLICITACAO'],
                "IDSITUACAO"                => $parametros['IDSITUACAO'],
                "IDTIPOHISTORICO"           => 1,
                "NRUSUARIOAPPSOLICITANTE"   => $usuario,
                "DSSOLICITACAO"             => 'Solicitação criada por um cliente externo.',
                "IDSATISFACAOSOLICITA"      => null,
                "NRORG"                     => $NRORG,
            ));

            if (count($parametros['IMAGENS']) > 0) {
                $row = $this->FileServerService->uploadFiles($parametros['IMAGENS'], 'anexosSolicitacoes');

                foreach ($row as $file) {
                    $this->SolicitacaoFactory->insertImg(array(
                        "NRIMGSOLICITACAO" => $this->newCode->getCodeByTableName('FCM_IMGSOLICITACAO', $this->environment->getNrOrg(), 1, 10, true),
                        "NRSOLICITACAO" => $NRSOLICITACAO,
                        "DSCAMINHO" => $file['path'],
                        "NRORG" => $NRORG,
                        "DSTYPE" => $file['type'],
                        "NRSIZE" => $file['size']
                    ));
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function avaliarSolicitacao($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $this->SolicitacaoFactory->upRequest(array(
                "NRSOLICITACAO"        => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"           => ($parametros['CONDITION']) ? 4 : 1,
                "DSSOLICITACAO"        => $parametros['DSSOLICITACAO']
            ));

            $this->SolicitacaoFactory->insertRequestHistory(array(
                "NRSOLICITACAOHISTORICO" => $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true),
                "NRSOLICITACAO"          => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"             => ($parametros['CONDITION']) ? 4 : 1,
                "IDTIPOHISTORICO"        => ($parametros['CONDITION']) ? 4 : 1,
                "CDOPERSOLICITANTE"      => $cdoperador,
                "DSSOLICITACAO"          => ($parametros['CONDITION']) ? 'Solicitação Avaliada.' : 'Solicitação recusada pelo solicitante.',
                "IDSATISFACAOSOLICITA"   => $parametros['IDSATISFACAOSOLICITA'],
                "NRORG"                  => $NRORG,
                "CDOPERULTATU"           => $cdoperador,
            ));

            if (!($parametros['CONDITION'])) {
                // atualizar tarefa na checklistagend
                $dados = $this->SolicitacaoFactory->getTarefasVinculadas(array(
                    "NRSOLICITACAO"        => $parametros['NRSOLICITACAO'],
                ));

                $this->SolicitacaoFactory->setStatusTarefas(array(
                    "NRCHECKLISTAGEND"        => $dados[0]['NRCHECKLISTAGEND'],
                    "IDSITUACAOAGEND"         => ($parametros['CONDITION']) ? 4 : 3,
                    "CDOPERULTATU"            => $cdoperador
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // CODIGO TELA REFATORADA
    public function getSolicitacoes($params)
    {

        $session = InstanceManager::getInstance()->getService('session');
        $session->start();
        $usuario = $session->get('NRUSUARIOAPP');

        $filter = "";

        if ($params['TIPO'] == "2") {
            if (isset($params['NRLOCALCLIENTE'])) {
                $local = $this->FindService->formatarArray($params['NRLOCALCLIENTE']);
                $filter =  $filter . " AND S.NRLOCALCLIENTE IN (" . $local . ")";
            }

            if (isset($params['DTHORAAGEND'])) {
                $filter = $filter . " AND DTSOLICITACAO BETWEEN'" . $params['DTHORAAGEND'][0] . "' AND '" . $params['DTHORAAGEND'][1] . "'";
            }

            if (isset($params['IDSITUACAO'])) {
                $situacao = $this->FindService->formatarArray($params['IDSITUACAO']);
                $filter = $filter . " AND IDSITUACAO IN (" . $situacao . ")";
            }

            if (isset($params['NRSOLICITACAO'])) {
                $solicitacao = $this->FindService->formatarArray($params['NRSOLICITACAO']);
                $filter = $filter . " AND S.NRSOLICITACAO IN (" . $solicitacao . ")";
            }
        }

        return $this->SolicitacaoFactory->getSolicitacoes($usuario, $filter);
    }

    public function getFotosSolicitacao($params)
    {
        $dados = $this->SolicitacaoFactory->getFotosSolicitacao($params);
        foreach ($dados as &$value) {
            $value['DSCAMINHO'] = $this->FileServerService->fileServerUrl . '/files/'. $value['DSCAMINHO'];
        }
        return $dados;
    }

    public function getAvaliacao($parametros)
    {
        return $this->SolicitacaoFactory->getAvaliacao(array(
            "NRSOLICITACAO" => $parametros['NRSOLICITACAO']
        ));
    }
}
